package net.sf.sql2java.conversion;

import java.util.HashMap;
import java.util.Map;

/**
 *
 * @author dge2
 */
public class KeywordGroup {
	public static final String JAVA = "java";
	public static final String SQL 	= "sql";
	
    private String name;
    private String transform;
    
    private boolean caseSensitive;
    
    private Map<String, Keyword> keywords = new HashMap<String, Keyword>();
    
// -- Keyword Translation -----------------------------------------------------
    
    public String translate(String input) {
        Keyword keyword = getKeyword(input);
        
        if (keyword == null )
            return input;
        
        return keyword.translate();
    }

// -- Keyword List Manipulation -----------------------------------------------
    
    public void addKeyword(Keyword keyword) {
        if (keywords == null)
            keywords = new HashMap<String, Keyword>();
        
        keyword.setGroup(this);
        
        keywords.put(keyword.getName(), keyword);
    }
    
    public void removeKeyword(Keyword keyword) {
        if (keywords != null)
            keywords.remove(keyword.getName());
    }
    
    public Keyword getKeyword(String input) {
        if (caseSensitive) {
            return keywords.get(input); 
        } else {
            for (String key : keywords.keySet())
                if (key.equalsIgnoreCase(input))
                    return keywords.get(key);
            
            return null;
        }
    }
    
// -- Getters -----------------------------------------------------------------
    
    public String getName() {
        return name;
    }
    
    public String getTransform() {
        return transform;
    }
    
    public boolean isCaseSensitive() {
        return caseSensitive;
    }
    
    public Map<String, Keyword> getKeywords() {
    	return keywords;
    }
    
// -- Setters -----------------------------------------------------------------
    
    public void setName(String name) {
        this.name = name;
    }
    
    public void setTransform(String transform) {
        this.transform = transform;
    }
    
    public void setCaseSensitive(boolean caseSensitive) {
        this.caseSensitive = caseSensitive;
    }
    
    public void setKeywords(Map<String, Keyword> keywords) {
    	this.keywords = keywords;
    }
}
