package net.sf.sql2java.generator.merger;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import net.sf.sql2java.common.parameters.StringParser;
import net.sf.sql2java.common.parameters.handlers.SimpleMapParameterHandler;
import net.sf.sql2java.generator.writer.merge.Conflict;

import org.apache.log4j.Logger;

public class ExternalMerger extends AbstractMerger {
	private static final Logger LOGGER = Logger.getLogger(ExternalMerger.class);
	
	public enum Props { executable, arg, persistConflicts }
	public enum Parameters { od, gd, op, gp }

// -- Merging -----------------------------------------------------------------
	
	@Override
	public void merge(Conflict conflict) throws Exception {
		// -- Log the whole process
		LOGGER.info(String.format("Merging Conflict on %s", conflict.getOriginalFile().getAbsolutePath()));
		
		// -- execute the command
		executeProcess(parseCommand(getCommand(), conflict));
	}
	
	@Override
	public void cleanup(Set<Conflict> conflicts) throws Exception {
		// -- remove the conflict directory if needed
		if (! getPersistConflicts()) {
			for (Conflict conflict : conflicts) {
				if (conflict.getGeneratedFile().exists()) {
					boolean isDeleted = conflict.getGeneratedFile().delete();
					if (! isDeleted) {
						LOGGER.warn(String.format(
								"Unable to delete %s!", 
								conflict.getGeneratedFile().getAbsolutePath())
						);
					}
				}
			}
		}
	}
	
	protected void executeProcess(String[] command) throws Exception {
		// -- create the process
		Process process = Runtime.getRuntime().exec(command);
		
		// -- wait for the process to finish
		process.waitFor();
	}
	
	protected String[] parseCommand(List<String> command, Conflict conflict) throws Exception {
		// -- create the context
		Map<String, String> context = new HashMap<String, String>();
		context.put(Parameters.od.toString(), "\"Original\"");
		context.put(Parameters.gd.toString(), "\"Generated\"");
		context.put(Parameters.op.toString(), conflict.getOriginalFile().getCanonicalPath());
		context.put(Parameters.gp.toString(), conflict.getGeneratedFile().getCanonicalPath());
		
		StringParser parser = new StringParser(new SimpleMapParameterHandler(context));
		
		List<String> result = new ArrayList<String>();
		for (String item : command)
			result.add(parser.parse(item));
		
		return result.toArray(new String[result.size()]);
	}
	
// -- Getters -----------------------------------------------------------------
	
	public List<String> getCommand() {
		List<String> result = new ArrayList<String>();
		
		// -- add the executable
		result.add(getProperty(Props.executable.toString()));

		// -- add all parameters
		for (Object key : getProperties().keySet())
			if (key.toString().startsWith(Props.arg.toString()))
				result.add(getProperty(key.toString()));
		
		return result;
	}
	
	public boolean getPersistConflicts() {
		String property = getProperty(Props.persistConflicts.toString(), "false");
		
		return property.equalsIgnoreCase("true");
	}
}
