package net.sf.sql2java.generator.template.pack;

import java.util.HashMap;
import java.util.Map;

import net.sf.sql2java.common.configuration.Configurer;
import net.sf.sql2java.common.store.PackStore;

import org.apache.commons.configuration.Configuration;
import org.apache.commons.configuration.ConfigurationException;
import org.apache.log4j.Logger;

public class TemplatePackConfigurer extends Configurer<PackStore> {
	private static final Logger LOG = Logger.getLogger(TemplatePackConfigurer.class);

// -- Configuration -----------------------------------------------------------	

	@Override
	public void configure(PackStore subject) throws ConfigurationException {
		LOG.info("Configuring packStore.");
		
		// -- check if the configuration is valid
		if (getConfiguration() == null)
			throw new ConfigurationException("No configuration has been set");
		
		// -- get the packs
		subject.setPacks(getPacks(getConfiguration().subset("generator")));
	}

// -- Helpers -----------------------------------------------------------------
	
	protected Map<String, TemplatePack> getPacks(Configuration config) {
		// -- create the result
		Map<String, TemplatePack> result = new HashMap<String, TemplatePack>();
		
		int size = config.getList("packages.pack[@resource]").size();
		for (int i = 0; i < size; i++) {
			// -- create the template pack
			TemplatePack pack = readTemplatePack(
					config.subset(String.format("packages.pack(%d)", i)),
					config.getString(String.format("packages.pack(%d)[@resource]", i))
			);
			
			// -- add the template pack
			result.put(pack.getName(), pack);
			
			LOG.info(String.format("found new pack '%s'", pack.getName()));
		}
		
		// -- return the result
		return result;
	}
	
	protected TemplatePack readTemplatePack(Configuration config, String resource) {
		TemplatePack result = null;
		
		// -- create the packageReader
        PackageReader packageReader;
        
        // -- instantiate the right packageReader depending on the extension
        if (resource.endsWith(".jar")) {
        	throw new RuntimeException("Jar template packs are not supported yet!");
        } else {
            packageReader = new DirectoryPackageReader();
        }
        
        try {
        	// -- create the template pack
        	result = new TemplatePack();
        	
        	// -- read the properties
            result.setProperties(readProperties(config));
            
            // -- read the package
            packageReader.readPack(result, resource);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
		
		// -- return the result
		return result;
	}
	
	protected Map<String, String> readProperties(Configuration config) {
		// -- create the result
		Map<String, String> result = new HashMap<String, String>();
		
		int propertySize = config.getList("property[@name]").size();
		for (int i = 0; i < propertySize; i++) {
			// -- get the property key and value
			String key = config.getString(String.format("property(%d)[@name]", i));
			String value = config.getString(String.format("property(%d)[@value]", i));
			
			// -- set the property
			result.put(key, value);
		}

		// -- return the result
		return result;
	}
	
}
