/*
 * SwingUtilities.java
 *
 * Created on 20 oktober 2007, 14:47
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package net.sf.sql2java.utils;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GradientPaint;
import java.awt.Graphics2D;
import java.awt.Toolkit;
import java.awt.image.BufferedImage;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import org.apache.log4j.Logger;

/**
 *
 * @author dge2
 */
public class SwingUtilities {
    private static final Logger LOGGER = Logger.getLogger(SwingUtilities.class.getName());
    public static final String ICON_PATH = "resources/icons";
    
    public static void centerOnScreen(Component component) {
        // -- get the screen size
        Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
        
        // -- calculate component location
        int x = ((dim.width - component.getWidth())) / (2);
        int y = ((dim.height - component.getHeight())) / (2);
        
        // -- set the location of the component
        component.setLocation(x, y);
    }
 
    public static BufferedImage createReflection(BufferedImage image) {
        int height = image.getHeight();
        
        BufferedImage result = new BufferedImage(image.getWidth(), (int) (height * 1.5), BufferedImage.TYPE_INT_ARGB);
        Graphics2D g2 = result.createGraphics();
        
        // -- paint the original image
        g2.drawImage(image, 0, 0, null);
        
        // -- paint the mirrored image
        g2.scale(1.0,-1.0);
        g2.drawImage(image, 0, -height-height, null);
        g2.scale(1.0, -1.0);
        
        // -- move to the origin of the clone
        g2.translate(0, height);
        
        // -- create the alpha mask
        GradientPaint mask;
        mask = new GradientPaint(0, 0, new Color(1.0f, 1.0f, 1.0f, 0.5f), 0, ((float) height) / 2.0f, new Color(1.0f, 1.0f, 1.0f, 0.0f));
        g2.setPaint(mask);
        
        // -- set the alpha composite
        g2.setComposite(AlphaComposite.DstIn);
        
        // -- paint the mask
        g2.fillRect(0, 0, image.getWidth(), height);
        
        // -- cleanup
        g2.dispose();
        
        // -- return the image
        return result;
    }

    public static Icon getIcon(String iconName) {
        // -- create the result
        ImageIcon result = null;
        
        // -- add the iconpath to the iconName
        if (! iconName.startsWith("/")) iconName = "/" + iconName;
        iconName = ICON_PATH + iconName;
        
        try {
            result = new ImageIcon(iconName);
        } catch (Exception e) {
            LOGGER.warn(e.getMessage());
        }
        
        // -- return the result
        return result;
    }
}
